@@ -304,6 +304,12 @@
 /* .IP "\fBsmtp_tls_secure_cert_match (nexthop, dot-nexthop)\fR"
 /*	The server certificate peername verification method for the
 /*	\fBsecure\fR TLS security level.
+/* .IP "\fBsmtp_tls_security_level (empty)\fR"
+/*	The default SMTP TLS security level for all destinations, this
+/*	parameter obsoletes the legacy smtp_use_tls, smtp_enforce_tls,
+/*	and smtp_tls_enforce_peername parameters, which are used when
+/*	smtp_tls_security_level is left empty, and are ignored when
+/*	smtp_tls_security_level is not empty.
 /* .IP "\fBsmtp_tls_session_cache_database (empty)\fR"
 /*	Name of the file containing the optional Postfix SMTP client
 /*	TLS session cache.
@@ -318,7 +324,7 @@
 /*	process requests from the \fBtlsmgr\fR(8) server in order to seed its
 /*	internal pseudo random number generator (PRNG).
 /* .PP
-/*	Available in Postfix version 2.3 and later:
+/*	Available in Postfix version 2.4 and later:
 /* .IP "\fBsmtp_sasl_tls_verified_security_options ($smtp_sasl_tls_security_options)\fR"
 /*	The SASL authentication security options that the Postfix SMTP
 /*	client uses for TLS encrypted SMTP sessions with a verified server
@@ -552,6 +558,7 @@
 #include <msg.h>
 #include <mymalloc.h>
 #include <name_mask.h>
+#include <name_code.h>
 
 /* Global library. */
 
@@ -625,6 +632,7 @@ bool    var_smtp_cache_demand;
 char   *var_smtp_ehlo_dis_words;
 char   *var_smtp_ehlo_dis_maps;
 
+char   *var_smtp_tls_level;
 bool    var_smtp_use_tls;
 bool    var_smtp_enforce_tls;
 char   *var_smtp_tls_per_site;
@@ -678,6 +686,8 @@ SSL_CTX *smtp_tls_ctx;
 
 #endif
 
+extern NAME_CODE smtp_tls_levels[];	/* smtp_session.c name_code table */
+
 /* deliver_message - deliver message with extreme prejudice */
 
 static int deliver_message(const char *service, DELIVER_REQUEST *request)
@@ -813,7 +823,10 @@ static void pre_init(char *unused_name, char **unused_argv)
     /*
      * Initialize the TLS data before entering the chroot jail
      */
-    if (var_smtp_use_tls || var_smtp_enforce_tls || var_smtp_tls_per_site[0]) {
+    if (name_code(smtp_tls_levels, NAME_CODE_FLAG_NONE,
+		  var_smtp_tls_level) > TLS_LEV_NONE ||
+	var_smtp_use_tls || var_smtp_enforce_tls ||
+	var_smtp_tls_per_site[0] || var_smtp_tls_policy[0]) {
 #ifdef USE_TLS
 	tls_client_init_props props;
 
