@@ -645,10 +645,19 @@ static int smtp_start_tls(SMTP_STATE *state)
      * multiple hosts per hostname, or even multiple hosts per IP address.
      * All this without a shared TLS session cache, and they still want to
      * use TLS session caching???
+     *
+     * The TLS session cache records the trust chain verification status
+     * of cached sessions. Different transports may have different CAfile
+     * or CApath settings, perhaps to allow authenticated connections to
+     * sites with private CA certs without trusting said private certs for
+     * other sites. So we cannot assume that a trust chain valid for one
+     * transport is valid for another. Therefore the client session id
+     * must include either the transport name or the values of CAfile and
+     * CApath. We use the transport name.
      */
     serverid = vstring_alloc(10);
-    vstring_sprintf(serverid, "%s:%u:%s", session->addr,
-		  ntohs(session->port), session->helo ? session->helo : "");
+    vstring_sprintf(serverid, "%s:%s:%u:%s", state->service, session->addr,
+		    ntohs(session->port), session->helo ? session->helo : "");
     /*
      * XXX: We store only one session per lookup key. Ideally the the
      * key maps 1-to-1 to a server TLS session cache. We use the
@@ -1087,14 +1096,18 @@ static int smtp_loop(SMTP_STATE *state, NOCLOBBER int send_state,
 	     */
 	case SMTP_STATE_XFORWARD_NAME_ADDR:
 	    vstring_strcpy(next_command, XFORWARD_CMD);
-	    if (session->features & SMTP_FEATURE_XFORWARD_NAME)
-		vstring_sprintf_append(next_command, " %s=%s",
-		   XFORWARD_NAME, DEL_REQ_ATTR_AVAIL(request->client_name) ?
-			       request->client_name : XFORWARD_UNAVAILABLE);
-	    if (session->features & SMTP_FEATURE_XFORWARD_ADDR)
-		vstring_sprintf_append(next_command, " %s=%s",
-		   XFORWARD_ADDR, DEL_REQ_ATTR_AVAIL(request->client_addr) ?
-			       request->client_addr : XFORWARD_UNAVAILABLE);
+	    if (session->features & SMTP_FEATURE_XFORWARD_NAME) {
+		vstring_strcat(next_command, " " XFORWARD_NAME "=");
+		xtext_quote_append(next_command,
+				   DEL_REQ_ATTR_AVAIL(request->client_name) ?
+			   request->client_name : XFORWARD_UNAVAILABLE, "");
+	    }
+	    if (session->features & SMTP_FEATURE_XFORWARD_ADDR) {
+		vstring_strcat(next_command, " " XFORWARD_ADDR "=");
+		xtext_quote_append(next_command,
+				   DEL_REQ_ATTR_AVAIL(request->client_addr) ?
+			   request->client_addr : XFORWARD_UNAVAILABLE, "");
+	    }
 	    if (session->send_proto_helo)
 		next_state = SMTP_STATE_XFORWARD_PROTO_HELO;
 	    else
@@ -1103,20 +1116,26 @@ static int smtp_loop(SMTP_STATE *state, NOCLOBBER int send_state,
 
 	case SMTP_STATE_XFORWARD_PROTO_HELO:
 	    vstring_strcpy(next_command, XFORWARD_CMD);
-	    if (session->features & SMTP_FEATURE_XFORWARD_PROTO)
-		vstring_sprintf_append(next_command, " %s=%s",
-		 XFORWARD_PROTO, DEL_REQ_ATTR_AVAIL(request->client_proto) ?
-			      request->client_proto : XFORWARD_UNAVAILABLE);
-	    if (session->features & SMTP_FEATURE_XFORWARD_HELO)
-		vstring_sprintf_append(next_command, " %s=%s",
-		   XFORWARD_HELO, DEL_REQ_ATTR_AVAIL(request->client_helo) ?
-			       request->client_helo : XFORWARD_UNAVAILABLE);
-	    if (session->features & SMTP_FEATURE_XFORWARD_DOMAIN)
-		vstring_sprintf_append(next_command, " %s=%s", XFORWARD_DOMAIN,
+	    if (session->features & SMTP_FEATURE_XFORWARD_PROTO) {
+		vstring_strcat(next_command, " " XFORWARD_PROTO "=");
+		xtext_quote_append(next_command,
+				 DEL_REQ_ATTR_AVAIL(request->client_proto) ?
+			  request->client_proto : XFORWARD_UNAVAILABLE, "");
+	    }
+	    if (session->features & SMTP_FEATURE_XFORWARD_HELO) {
+		vstring_strcat(next_command, " " XFORWARD_HELO "=");
+		xtext_quote_append(next_command,
+				   DEL_REQ_ATTR_AVAIL(request->client_helo) ?
+			   request->client_helo : XFORWARD_UNAVAILABLE, "");
+	    }
+	    if (session->features & SMTP_FEATURE_XFORWARD_DOMAIN) {
+		vstring_strcat(next_command, " " XFORWARD_DOMAIN "=");
+		xtext_quote_append(next_command,
 			 DEL_REQ_ATTR_AVAIL(request->rewrite_context) == 0 ?
-				       XFORWARD_UNAVAILABLE :
+				   XFORWARD_UNAVAILABLE :
 		     strcmp(request->rewrite_context, MAIL_ATTR_RWR_LOCAL) ?
-				  XFORWARD_DOM_REMOTE : XFORWARD_DOM_LOCAL);
+			      XFORWARD_DOM_REMOTE : XFORWARD_DOM_LOCAL, "");
+	    }
 	    next_state = SMTP_STATE_MAIL;
 	    break;
 
