@@ -112,6 +112,7 @@
 #include <vstream.h>
 #include <stringops.h>
 #include <valid_hostname.h>
+#include <name_code.h>
 
 /* Global library. */
 
@@ -125,7 +126,14 @@
 #include "smtp.h"
 #include "smtp_sasl.h"
 
-/*#define msg_verbose 1*/
+NAME_CODE smtp_tls_levels[] = {
+    "none",    TLS_LEV_NONE,
+    "may",     TLS_LEV_MAY,
+    "encrypt", TLS_LEV_ENCRYPT,
+    "verify",  TLS_LEV_VERIFY,
+    "secure",  TLS_LEV_SECURE,
+    0, 	       TLS_LEV_NOTFOUND,
+};
 
 #ifdef USE_TLS
 
@@ -154,14 +162,11 @@ void    smtp_tls_list_init(void)
 
 static const char *policy_name(int level)
 {
-    switch (level) {
-    case TLS_LEV_SECURE: return ("secure");
-    case TLS_LEV_VERIFY: return ("verify");
-    case TLS_LEV_ENCRYPT: return ("encrypt");
-    case TLS_LEV_MAY: return ("may");
-    case TLS_LEV_NONE: return ("none");
-    default: return ("unknown");
-    }
+    const char *name = str_name_code(smtp_tls_levels, level);
+
+    if (name == 0)
+	name = "unknown";
+    return name;
 }
 
 /* tls_site_lookup - look up per-site TLS security level */
@@ -236,17 +241,8 @@ static int tls_policy_lookup_one(SMTP_SESSION *session, int *site_level,
 	FREE_RETURN (1);	/* No further lookups */
     }
 
-    if (!strcasecmp(tok, "none")) {
-	*site_level = TLS_LEV_NONE;
-    } else if (!strcasecmp(tok, "may")) {
-	*site_level = TLS_LEV_MAY;
-    } else if (!strcasecmp(tok, "encrypt")) {
-	*site_level = TLS_LEV_ENCRYPT;
-    } else if (!strcasecmp(tok, "verify")) {
-	*site_level = TLS_LEV_VERIFY;
-    } else if (!strcasecmp(tok, "secure")) {
-	*site_level = TLS_LEV_SECURE;
-    } else {
+    *site_level = name_code(smtp_tls_levels, NAME_CODE_FLAG_NONE, tok);
+    if (*site_level == TLS_LEV_NOTFOUND) {
 	msg_warn("%s: unknown security level '%s' ignored",
 		 str_context(cbuf, site_class, site_name), tok);
 	FREE_RETURN (1);	/* No further lookups */
@@ -346,12 +342,11 @@ static void tls_policy_lookup(SMTP_SESSION *session, int *site_level,
 
 /* session_tls_init - session TLS parameters */
 
-static void session_tls_init(SMTP_SESSION *session,
-				const char *dest, const char *host)
+static void session_tls_init(SMTP_SESSION *session, const char *dest,
+				 const char *host, int flags)
 {
     int     global_level;
     int     site_level;
-    int     tls_level;
 
     /*
      * Initialize all TLS related session properties.
@@ -369,12 +364,22 @@ static void session_tls_init(SMTP_SESSION *session,
      * no per-site policy exists. It also is used to override a wild-card
      * per-site policy.
      */
-    if (var_smtp_enforce_tls)
+    if (*var_smtp_tls_level) {
+	global_level = name_code(smtp_tls_levels, NAME_CODE_FLAG_NONE,
+				 var_smtp_tls_level);
+	if (global_level == TLS_LEV_NOTFOUND) {
+	    msg_fatal("%s: unknown TLS security level '%s'",
+		      (flags & SMTP_MISC_FLAG_USE_LMTP) ?
+			  VAR_LMTP_TLS_LEVEL : VAR_SMTP_TLS_LEVEL,
+		      var_smtp_tls_level);
+	}
+    } else if (var_smtp_enforce_tls) {
 	global_level = var_smtp_tls_enforce_peername ?
 	    TLS_LEV_VERIFY : TLS_LEV_ENCRYPT;
-    else
+    } else {
 	global_level = var_smtp_use_tls ?
 	    TLS_LEV_MAY : TLS_LEV_NONE;
+    }
     if (msg_verbose)
 	msg_info("%s TLS level: %s", "global", policy_name(global_level));
 
@@ -502,7 +507,7 @@ SMTP_SESSION *smtp_session_alloc(VSTREAM *stream, const char *dest,
      * security levels are defined.
      */
 #ifdef USE_TLS
-    session_tls_init(session, dest, host);
+    session_tls_init(session, dest, host, flags);
 #endif
     session->state = 0;
     debug_peer_check(host, addr);
