@@ -268,7 +268,7 @@ static void uncache_session(TLScontext_t *TLScontext)
 
 SSL_CTX *tls_client_init(const tls_client_init_props *props)
 {
-    int     off = 0;
+    long    off = 0;
     SSL_CTX *client_ctx;
     int     cachable;
 
