@@ -243,7 +243,7 @@ static int new_server_session_cb(SSL *ssl, SSL_SESSION *session)
 
 SSL_CTX *tls_server_init(const tls_server_props *props)
 {
-    int     off = 0;
+    long    off = 0;
     int     verify_flags = SSL_VERIFY_NONE;
     SSL_CTX *server_ctx;
     int     cachable;
